<?php
// $Id: simple_contest.api.inc,v 1.1.2.1 2010/07/09 04:36:13 vinoth.3v Exp $


/**
 * Returns the simple_contest settings
 * @param $type The content type to return settings for
 * @return array
 * simple contest settigns for the content type $type 
 */
function simple_contest_get_settings($type, $refresh = FALSE) {
  static $settings; 
  
  $default_settings = array();
  
  if ($refresh) {
    $default_settings = module_invoke_all('simple_contest', 'default_settings');
  } else if (isset($settings[$type])) {
    return $settings[$type];
  }
  
  $new_settings = variable_get('simple_contest_'. $type, $default_settings);
  
  if (!empty($default_settings)){
  	$new_settings += $default_settings;
  }
  
  $settings[$type] = $new_settings;
  
  return $new_settings;
}

/**
 * Function simple_contest_finalize_results_all
 * 
 */
function simple_contest_finalize_results_all(){

  $fields = content_fields();

  //$today = date('Y-m-d', time());
	
  foreach (simple_contest_get_types() as $type => $info) {

    $settings = simple_contest_get_settings($type);

    if(empty($settings['date_field'])){
      watchdog('simple_contest', 'Date field for the contest %type is not set.'  , array('%type' => $type),  WATCHDOG_WARNING);
      continue;
    }

    // process only for automatic winner selection types
    if($settings['winner_selection_type'] != 'auto'){
      continue;
    }

    $field = $fields[$settings['date_field']];
    $db_info = content_database_info($field);
    
    
    $table = $db_info['table'];
    $column_name = '';

    foreach ($db_info['columns'] as $column_name => $attributes) {
      $column_name = $db_info['columns'][$column_name]['column'];
      break;
    }
		
		// skip if db info is missing
		if ($column_name == '') {
			continue;
		}
		
		// time zone handling for the date field.
    $def_tz = date_get_timezone($field['tz_handling']);
		
		$def_tz_db = date_get_timezone_db($def_tz);
		
		$site_date = date_make_date(time(), $def_tz_db, DATE_UNIX);
		date_timezone_set($site_date, timezone_open($def_tz_db));
		
		$site_date_string = date_format_date($site_date, 'custom', 'Y-m-d\TH:i:s');
		$site_time = strtotime($site_date_string);
		
		
    $column_name2 = $column_name . '2'; // todate column from date module

    $sql = "SELECT n.nid, " . $column_name .", " . $column_name2 ."
			FROM {node} n 
			LEFT JOIN {" . $table . "} ct ON n.nid = ct.nid AND n.vid = ct.vid 
			LEFT JOIN {simple_contest_statistics} sc ON n.nid = sc.nid 
			WHERE n.status <> 0 AND n.type = '%s' AND  
				UNIX_TIMESTAMP(" . $column_name2 .") < %d 
				AND sc.status = %d
			GROUP BY n.nid ORDER BY " . $column_name ." 
			";

    $res = db_query($sql, $type, $site_time, SIMPLE_CONTEST_STATUS_ACTIVE);

    while($row = db_fetch_object($res)){

      simple_contest_finalize_results_node(node_load($row->nid));
       
    }
     
  }
}

function simple_contest_finalize_results_node(&$node, $settings = NULL){

  $nid = $node->nid;
  $type = $node->type;

  if(!$settings){
    $settings = simple_contest_get_settings($type);
  }

  $view = views_get_view($settings['view']);

  if(!$view){
    watchdog('simple_contest', 'Cannot find the view %view for the contest %type.'  , array('%view' => $settings['view'], '%type' => $type),  WATCHDOG_WARNING);
    return array();
  }
  
  $date_field = $settings['date_field'];
  $arg1 = $node->{$date_field}[0]['value'];
  $arg2 = $node->{$date_field}[0]['value2'];
  
  $args = array($nid, $arg1, $arg2); // views must have two atleast null arguments

  $view->init_display();
  $view->display_handler->set_option('items_per_page', $settings['max_winners']);
  $view->display_handler->set_option('offset', 0);
  $view->preview('default', $args);

  $winners = array();

  if (empty($view->result) || !is_array($view->result)) {
    return $items;
  }

  $count = 0;
  //$field = ($settings['winner_type'] == 'user') ? 'uid' : 'nid';
  $field = $settings['view_field'];
  foreach ($view->result as $i => $row) {
    if(!isset($row->$field)){
      watchdog('simple_contest', 'The view must have the field ' . $field . ' to select winner for the contest ' . $type,  WATCHDOG_WARNING);
      break;
    }
    $winners[] = $row->$field;
    if($count >= $settings['max_winners']){
      break;
    }

  }

  simple_contest_save_result($node, $winners);

}

function simple_contest_save_result(&$node, $winners){

  global $language;
  $settings = simple_contest_get_settings($node->type);

  // delete the old
  $sql = "DELETE FROM {simple_contest_results} WHERE nid = %d";
  db_query($sql, $node->nid);

  $sql = "UPDATE {simple_contest_statistics} SET status = %d, winners_count = %d, timestamp = %d WHERE nid = %d";
  db_query($sql, SIMPLE_CONTEST_STATUS_COMPLETED, count($winners), time(), $node->nid);

  $new_winners = array();

  if ($settings['winner_type'] == 'user'){ //
    $weight = 0;
    foreach($winners as $uid){ // to keep the weight
      $new_winners[$uid] = array('nid' => NULL, 'weight' => $weight);
      $weight++; //rank
    }
  }else if ($settings['winner_type'] == 'node'){
    // find the user ids
    $nids = implode(',', $winners);
    $sql = 'SELECT nid, uid FROM {node} where nid in (' . $nids . ')';
    $res = db_query($sql);

    while($row = db_fetch_object($res)){
      $tmp_winners[$row->nid] = $row->uid;
    }
    $weight = 0;
    foreach($winners as $nid){ // to keep the weight
      $uid = 0;
      if(isset($tmp_winners[$nid])){
        $uid = $tmp_winners[$nid];
      }
      $new_winners[$uid] = array('nid' => $nid, 'weight' => $weight);
      $weight++; //rank
    }
  }

  //lets others to change it
  drupal_alter('simple_contest_results', $node, $new_winners);

  $sql = "INSERT INTO {simple_contest_results} (nid, uid, wnid, weight) VALUES (%d, %d, %d, %d)";
  foreach($new_winners as $uid => $info){
    db_query($sql, $node->nid, $uid, $info['nid'], $info['weight']);
  }

  //invoke hooks
  module_invoke_all('simple_contest', SIMPLE_CONTEST_STATUS_COMPLETED, $node, $new_winners);

  if($settings['email_to_winner']) {
    foreach($new_winners as $uid => $info){

      $account = user_load(array('uid' => $uid));
      if ($info['nid']){
        $participant_node = node_load($info['nid']);
      }else{
        $participant_node = NULL;
      }
      
      $params = array(
        'contest_node' => $node, 
        'participant_user' => $account, 
        'participant_node' => $participant_node, 
        'settings' => $settings, 
        'participant_rank' => $info['weight'], 
      );
      
      drupal_mail('simple_contest', 'winners_mail', $account->mail, $language, $params);
      
    }
  }
  
  //TODO: email_to_participants
  //Create a new view to get all participants. it must has nid if node contest, uid, uname, user email fields of complete participants
  if($settings['email_to_participants']){
    $participants = simple_contest_get_participants_from_view($node, $settings);
    
    //???: batch process?
  
    foreach($participants as $data){
      
      if(isset($new_winners[$data['uid']])){ // s/he is winner
        continue;
      }
      
      $account = user_load(array('uid' => $data['uid']));
      $participant_node = NULL;
      
      if ($settings['winner_type'] == 'node'){
        $participant_node = node_load($data['nid']);
      }
      
      $params = array(
		'contest_node' => $node, 
		'participant_user' => $account, 
		'participant_node' => $participant_node, 
		'settings' => $settings, 
		'participant_rank' => -1, // TODO: fix
      );
      
      drupal_mail('simple_contest', 'winners_mail_participants', $account->mail, $language, $params);
    }
  }
  
  //E-mail to admin
  if($settings['email_to_admin']){
    $params = array(
      'contest_node' => $node, 
      'settings' => $settings, 
    );

    $admin = user_load(array('uid' => 1)); // ???: multiple admins?, site mail?
    drupal_mail('simple_contest', 'admin_winners_mail', $admin->mail, $language, $params);
  }

}

function simple_contest_get_node_result($node){
  $results = array();
  
  $settings = simple_contest_get_settings($node->type);
  
  $sql = "SELECT cr.uid, u.name, u.mail, cr.wnid, cr.weight 
			FROM {simple_contest_results} cr 
			JOIN {users} u ON u.uid = cr.uid 
			WHERE nid = %d ORDER BY weight";
  $res = db_query($sql, $nid);
  while($row = db_fetch_object($res)){
    $results[] = array(
		'nid' => $row->wnid,
		'uid' => $row->uid, 
		'mail' => $row->mail,
		'name' => $row->name,
		'weight' => $row->weight,
	);
  }

  return $results;

}


function simple_contest_get_participants_from_view(&$node, $settings = NULL){

  $participants = array();
  
  $nid = $node->nid;
  $type = $node->type;

  if(!$settings){
    $settings = simple_contest_get_settings($type);
  }

  $view = views_get_view($settings['email_to_participants_view']);

  if(!$view){
    watchdog('simple_contest', 'Cannot find the view %view for the contest %type to get the contest participants.'  , array('%view' => $settings['view'], '%type' => $type),  WATCHDOG_WARNING);
    return array();
  }

  //arguments to pass to view %1 = contest node ID, %2 = contest from date, %3 = contest to date 
  $args = array($nid, $node->{$settings['date_field']}[0]['value'], $node->{$settings['date_field']}[0]['value2']); // view must have two atleast null arguments

  $view->init_display();
  //$view->display_handler->set_option('items_per_page', $values['max_winners']);
  $view->display_handler->set_option('offset', 0);
  $view->preview('default', $args);

  if (empty($view->result) || !is_array($view->result)) {
    return $participants;
  }
  
  $vfielduid = $settings['email_to_participants_view_field_uid'];
  $vfieldnid = $settings['email_to_participants_view_field_nid'];
  
  foreach ($view->result as $idx => $row) {
    if($settings['winner_type'] == 'node' && (!isset($row->$vfielduid) || !isset($row->$vfielduid))) {
      watchdog('simple_contest', 'The view must have the fields uid and nid fields for the contest ' . $type,  WATCHDOG_WARNING);
    } else if ( !isset($row->$vfielduid)) {
      watchdog('simple_contest', 'The view must have the fields uid field for the contest ' . $type,  WATCHDOG_WARNING);
    }
    return $participants;
  }
  
  reset($view->result);
  
  //???: what about just returning $view->result ?
  if ($settings['winner_type'] == 'node') {
    foreach ($view->result as $idx => $row) {
      $participants[] = array(
      	'uid' => $row->$vfielduid, 
        'nid' => $row->$vfieldnid,
      );
    }
  } else {
    foreach ($view->result as $idx => $row) {
      $participants[] = array(
      	'uid' => $row->$vfielduid,
      );
    }    
  }
  
  return $participants;

}

/**
 * Function simple_contest_get_active_contests
 * Returns an array of active contest Node IDs of type ctype.
 *
 * @param unknown_type $ctype
 * @param unknown_type $refresh
 *
 * @return array
 * Returns an array of active contest Node IDs of type ctype.
 *
 */
function simple_contest_get_active_contests($ctype = NULL, $refresh = FALSE) {
  static $contests = array();

  if ($ctype && !simple_contest_is_simple_contest($ctype)) {
    return array();
  }

  if ($refresh) {
    if ($ctype) {
      unset($contests[$ctype]);
    } else {
      unset($contests['all']);
    }
  }

  if ($ctype && isset($contests[$ctype])) {
    return $contests[$ctype];
  }else if (isset($contests['all'])) {
    return $contests['all'];
  }

  $results = array();

  //???: should we consider node status here? what about active upcoming contest?
  if ($ctype) {
    $sql = "SELECT scs.nid FROM {simple_contest_statistics} scs
		INNER JOIN {node} n ON n.nid = scs.nid 
		WHERE scs.status = 1 AND n.status <> 0 AND n.type = '%s' ORDER BY n.title ";
    $res = db_query($sql, $ctype);
  } else {
    $sql = "SELECT scs.nid FROM {simple_contest_statistics} scs
		INNER JOIN {node} n ON n.nid = scs.nid 
		WHERE scs.status = 1 AND n.status <> 0 ORDER BY n.title ";
    $res = db_query($sql);
  }

  while($row = db_fetch_array($res)) {
    $results[$row['nid']] = $row['nid']; // node load ?
  }

  $contests[$ctype] = $results; // save to static

  return $results;

}



function simple_contest_status_name($status){
  static $options = array(
    SIMPLE_CONTEST_STATUS_INACTIVE => 'Disabled',
    SIMPLE_CONTEST_STATUS_ACTIVE => 'Active',
    SIMPLE_CONTEST_STATUS_CANCELED => 'Canceled',
    SIMPLE_CONTEST_STATUS_COMPLETED => 'Completed',
  );
  
  if (!isset($options[$status])){
    return '';
  }
  return $options[$status]; // t() not used here.

}

function simple_contest_node_winners_themed($node){
  $return = ''; 
  if(!isset($node->simple_contest['status'])){
    return $return;
  }
  
  $settings = simple_contest_get_settings($node->type);
  if($node->simple_contest['status'] == SIMPLE_CONTEST_STATUS_COMPLETED){
    $results_display_view = $settings['results_display_view'];
    $results = views_embed_view($results_display_view, 'default', $node->nid);

    $return = theme('simple_contest_results_display', $node, $results);
  }

  return $return;
}

/**
 * Helper function to get all fields of a view
 * ???: has any other way to get all fields?
 * 
 * @param unknown_type $view
 * @return array:
 * 
 */
function _simple_contest_get_view_fields($view) {
  $fields = array();
  $view = views_get_view($view);
  if ($view) {
    foreach($view->display['default']->display_options['fields'] as $id=>$field) {
      if($field['label']){
        $fields[$id] = t($field['label']);
      }
    }
  }
  return $fields;
}
