<?php
// $Id: simple_contest_repeat.form.inc,v 1.1.2.1 2010/07/09 04:36:13 vinoth.3v Exp $

/**
 * @file
 *
 * simple_contest_repeat custom module
 * mostly adopted from date/date/date_repeat.inc, thanks to karens.
 */

/**
 * Implementation of hook_form_FORM_ID_alter().
 */
function _simple_contest_repeat_content_type_settings_form_alter(&$form, &$form_state) {

  $type = $form['type']['#value'];

  $settings = simple_contest_get_settings($type);

  module_load_include('inc', 'simple_contest_repeat', 'simple_contest_repeat.helper');
  $def_node = simple_contest_repeat_get_def_node($type, TRUE); // always refresh here?
  
  $form['schedule'] = array(
		'#type' => 'fieldset',
		'#title' => t('Contest scheduler'),
		'#collapsible' => TRUE,
		'#collapsed' => !$settings['auto_create'],
		'#description' => t("You can set repeat rules below to schedule the creation of contests automatically. Default node is the 'Unpublished' template node which will be used when creating scheduled contests. 
    The default node template for this contest is !node.", array('!node' => l($def_node->title, 'node/' . $def_node->nid))),
        '#weight' => 5,
  );
  
  $form['schedule']['auto_create'] = array(
		'#type' => 'checkbox',
		'#title' => t('Enable contest auto create using repeat rule.'),
		'#default_value' => $settings['auto_create'],
		'#options' => array(0 => 0, 1 => 1),
		'#required' => FALSE,
  );
  
  // time zone
  $tz = '';
  if($settings['tz_handling'] != ''){
    $tz = date_get_timezone_db($settings['tz_handling']);
  }

  $form['schedule']['rrule'] = array(
	'#title' => t('Repeat rule'),
    '#type' => 'date_repeat_rrule',
    '#default_value' => isset($settings['rrule']['rrule']) ? $settings['rrule']['rrule'] : '',
    '#date_timezone' => $tz,
    '#date_format'      => date_limit_format('Y-m-d', array('year', 'month', 'day')),
    '#date_text_parts'  => array(),
    '#date_increment'   => 1,
    '#date_year_range'  => '-3:+3',
    '#date_label_position' => 'within',
    '#prev_value' => isset($settings['rrule']['value']) ? $settings['rrule']['value'] : '',
    '#prev_value2' => isset($settings['rrule']['value2']) ? $settings['rrule']['value2'] : '',
    '#prev_rrule' => isset($settings['rrule']['rrule']) ? $settings['rrule']['rrule'] : '',
    '#date_repeat_widget' => module_exists('date_popup') ? 'date_popup' : 'date_select',
    '#date_repeat_collapsed' => FALSE,
  );

  $options = array(
		'site' => t("Site's time zone"),
		'date' => t("Date's time zone"),
		'user' => t("User's time zone"),
		'utc' => 'UTC',
		'' => t('No time zone conversion'),
  );

  $form['schedule']['tz_handling'] = array(
		'#type' => 'select',
		'#title' => t('Time zone handling'),
		'#default_value' => $settings['tz_handling'],
		'#options' => $options,
		'#description' => t('Select the timezone handling method to be used for this date.'),
  );
  
  $form['schedule']['contest_defaults_nid'] = array(
    '#value' => $def_node->nid,
    '#type' => 'hidden',
  );

  $form['#validate'][] = 'simple_contest_repeat_date_repeat_widget_validate';
  $form['#submit'][] = 'simple_contest_repeat_date_repeat_widget_submit';

}

/**
 * Validation for date repeat form element.
 *
 * Create multiple values from the RRULE results.
 * TODO: Lots more work needed here.
 */
function _simple_contest_repeat_date_repeat_widget_validate($form, &$form_state) {

  $form_values = $form_state['values'];

  if (!$form_values['auto_create']) {
    return;
  }

  module_load_include('inc', 'date_repeat', 'date_repeat_form');
  module_load_include('inc', 'date_api', 'date_api_ical');

  $form_state['values']['rrule'] = $form['#post']['schedule']['rrule'];

  if ($form_values['FREQ'] == 'NONE') {
    $error_field = 'schedule][rrule][FREQ';
    form_set_error($error_field, t('The frequency is required for repeating contests.'));
  }

  if (empty($form_values['UNTIL']['datetime'])) {
    $error_field = 'schedule][rrule][UNTIL][datetime][date';
    form_set_error($error_field, t('The UNTIL value is required for repeating contests.'));
  }
  
  if (form_get_errors()) {
    return;
  }

  $rrule = date_api_ical_build_rrule($form_values);
  $form_state['values']['rrule'] = date_ical_parse($rrule);
  $form_state['values']['rrule']['rrule'] = $rrule;

}

/**
 * Validation for date repeat form element.
 *
 * Create multiple values from the RRULE results.
 * Lots more work needed here.
 */
function _simple_contest_repeat_date_repeat_widget_submit($form, &$form_state) {

  $form_values = $form_state['values'];
  $type = $form_values['type'];
  
  // delete all nulls for this contest type
  $sql = "DELETE FROM {simple_contest_repeat}
		WHERE type = '%s' AND nid IS NULL ";
  db_query($sql, $type);

  if(!$form_values['auto_create']){
    return;
  }

  module_load_include('inc', 'date_repeat', 'date_repeat_form');
  module_load_include('inc', 'date_api', 'date_api_ical');

  $rrule = $form_state['values']['rrule']['rrule'];

  //$values -- from & to date?
  //$type = $settings['type'];
  $settings = simple_contest_get_settings($type);
  $fields = content_fields();
  $field = $fields[$settings['date_field']];
  $values = $form_values;
  
  module_load_include('inc', 'simple_contest_repeat', 'simple_contest_repeat.helper');
  simple_contest_repeat_date_repeat_build_dates($rrule, $settings, $values, $field, $item);

}

