<?php
// $Id: simple_contest_repeat.helper.inc,v 1.1.2.1 2010/07/09 04:36:13 vinoth.3v Exp $

/**
 * @file simple_contest_repeat.helper.inc
 *
 */

/**
 * Function simple_contest_repeat_create_contests.
 *
 */
function simple_contest_repeat_create_contests() {

  foreach (simple_contest_get_types() as $type => $info) {

    $settings = simple_contest_get_settings($type);

    if ($settings['auto_create'] == 1 && !empty($settings['rrule']) && $settings['contest_defaults_nid']){

      $limit = 1;
      
      $timezone = variable_get('date_default_timezone', 0);
      
      $nextday = date('Y-m-d', strtotime("+1 day"));
      
      $sql = "SELECT * FROM {simple_contest_repeat} 
			WHERE nid IS NULL AND type = '%s' AND DATE_FORMAT(start_date, '%Y-%m-%d') <= '". $nextday ."' 
			ORDER BY start_date ASC 
			LIMIT %d ";
      
      $res = db_query($sql, $type, $limit);
      while($row = db_fetch_object($res)){
        
        $def_node = node_load($settings['contest_defaults_nid']);
        $date_field = $settings['date_field'];

        if($def_node) {
          unset($def_node->nid);          
        } else {
          watchdog('simple_contest', 'Cannot find the default node for %type. Please goto the simple contest admin page and save again.', array('%type' => $type),  WATCHDOG_WARNING);
          $def_node = new stdClass();
          $def_node->title = "Contest - !start - !end";
          $def_node->type = $type;
        }

        $def_node->title = t($def_node->title, array('!start' => $row->start_date, '!end' => $row->end_date));
        $def_node->is_new = TRUE;
        $def_node->status = 1; // published

        $def_node->{$date_field}[0]['value'] = $row->start_date;
        $def_node->{$date_field}[0]['value2'] = $row->end_date;

        node_save($def_node); // save the new node

        db_query("UPDATE {simple_contest_repeat} SET nid = %d
					WHERE rid = %d " , $row->rid);
      }
    }
  }

}


/**
 * Helper function to build repeating dates from a $node_field.
 *
 * Pass in either the RRULE or the $form_values array for the RRULE,
 * whichever is missing will be created when needed.
 */
function simple_contest_repeat_date_repeat_build_dates($rrule, $settings, $values, $field, $item) {
  include_once('./'. drupal_get_path('module', 'date_api') .'/date_api_ical.inc');
  $field_name = $field['field_name'];

  // By the time we get here, the start and end dates have been
  // adjusted back to UTC, but we want localtime dates to do
  // things like '+1 Tuesday', so adjust back to localtime.

  $tz = $settings['tz_handling']=='' ? 'none' : $settings['tz_handling'];

  $timezone = date_get_timezone($tz, '');

  $timezone_db = date_get_timezone_db($tz);
  // start date is now ??
  //$start = date_make_date($item['value'], $timezone_db, $field['type'], $field['granularity']);
  $start = date_make_date(time(), $timezone_db, $field['type'], $field['granularity']);

  if ($timezone != $timezone_db){
    date_timezone_set($start, timezone_open($timezone));
  }

  $interval = max(1, isset($values['INTERVAL']) ? $values['INTERVAL'] : 1);

  switch ($values['FREQ']) {
    case 'DAILY':
      $jump = $interval .' days';
      break;
    case 'WEEKLY':
      $jump = $interval .' weeks';
      break;
    case 'MONTHLY':
      $jump = $interval .' months';
      break;
    case 'YEARLY':
      $jump = $interval .' years';
      break;
  }

  $end = drupal_clone($start);
  date_modify($start, $jump);

  $duration = date_difference($start, $end);
  $start_datetime = date_format($start, DATE_FORMAT_DATETIME);

  if (!empty($values['UNTIL']['datetime'])) {
    $end = date_ical_date($values['UNTIL'], $timezone);
  }
  $end_datetime = date_format($end, DATE_FORMAT_DATETIME);

  // Split the RRULE into RRULE and EXDATE parts.
  $parts = date_repeat_split_rrule($rrule);
  $parsed_exceptions = (array) $parts[1];
  $exceptions = array();
  foreach ($parsed_exceptions as $exception) {
    $date = date_ical_date($exception);
    $exceptions[] = date_format($date, 'Y-m-d');
  }

  // only the future date?
  //$dates = date_repeat_calc($rrule, $start_datetime, $end_datetime, $exceptions, $timezone);
  $dates = date_repeat_calc($rrule, $start_datetime, $end_datetime, $exceptions, $timezone);

  $sql = "INSERT INTO {simple_contest_repeat} (type, start_date, end_date) VALUES  ('%s', '%s', '%s')";

  $value = array();
  foreach ($dates as $delta => $date) {

    // date_repeat_calc always returns DATE_DATETIME dates, which is
    // not necessarily $field['type'] dates.
    // Convert returned dates back to db timezone before storing.
    $date_start = date_make_date($date, $timezone, DATE_DATETIME, $field['granularity']);
    date_timezone_set($date_start, timezone_open($timezone_db));
    $date_end = drupal_clone($date_start);
    date_modify($date_end, '+'. $duration .' seconds');
    $value[$delta] = array(
      'value' => date_format($date_start, date_type_format($field['type'])),
      'value2' => date_format($date_end, date_type_format($field['type'])),
      'offset' => date_offset_get($date_start),
      'offset2' => date_offset_get($date_end),
    //'timezone' => $timezone,
    //'rrule' => $rrule,
    );

    // insert into simple_contest_repeat
    db_query($sql, $settings['type'], $value[$delta]['value'], $value[$delta]['value2']);

  }
  return $value;
}

/*
 * default node template
 * TODO: refine this 'default_node_template' logic
 */
function simple_contest_repeat_get_def_node($type, $refresh){
  static $def_nodes = array();

  if (empty($type)){
    return;
  }

  if($refresh && isset($def_nodes[$type])){
    unset($def_nodes[$type]);
  }

  if(isset($def_nodes[$type]) && $def_nodes[$type]){
    return $def_nodes[$type];
  }

  $def_node = FALSE;
  $settings = simple_contest_get_settings($type);

  if ($settings['contest_defaults_nid']){
    $def_node = node_load($settings['contest_defaults_nid']);
  }

  if (!$def_node){

    $def_node = new stdClass();
    $def_node->is_new = TRUE;
    $def_node->type = $type;
    $def_node->title = "Contest - !start - !end"; // will be replaced on automatic contest creation
    $def_node->status = 0; // unpublished

    node_save($def_node); // save it

    $def_node = node_load($def_node->nid); // refresh?
    $settings['contest_defaults_nid'] = $def_node->nid;
    variable_set('simple_contest_' . $type, $settings);

  }

  $def_nodes[$type] = $def_node;

  return $def_node;
}
