<?php
// $Id: simple_contest_rules.rules.inc,v 1.1.2.4 2010/02/22 14:34:43 vinoth.3v Exp $

/**
 * @file
 * Provides rules integration
 */

/**
* Implementation of hook_rules_data_type_info()
*/
function simple_contest_rules_data_type_info() {
  return array(
    'contest_type' => array(
      'label' => t('Contest type'),
      'class' => 'simple_contest_rules_data_type_contest_type',
      'identifiable' => FALSE,
      'uses_input_form' => TRUE,
      'eval input' => TRUE,
    ),
/*    'contest_status' => array(
      'label' => t('Contest status'),
      'class' => 'simple_contest_rules_data_type_contest_status',
      'identifiable' => TRUE,
      'uses_input_form' => TRUE,
      'eval input' => TRUE,
    ),
*/
  );
}


/**
 * Implementation of hook_rules_event_info().
 */
function simple_contest_rules_event_info() {
  return array(
    'simple_contest_rules_contest_completed' => array(
       'label' => t('Contest was completed and winners were finalized'),
       'arguments' => array(
          'contest' => array('type' => 'node', 'label' => t('Contest node')),          
       ),
       'module' => 'Simple contest',
    ),
    'simple_contest_contest_status_changed' => array(
       'label' => t('Contest status was changed'),
       'arguments' => array(
          'contest' => array('type' => 'node', 'label' => t('Contest node')),
          'contest_author' => array('type' => 'user', 'label' => t('Contest author'), 'handler' => 'simple_contest_argument_load_contest_user'),
       ),
       'module' => 'Simple contest',
    ),
  );
}

function simple_contest_argument_load_contest_user($node){
	return user_load(array('uid' => $node->uid));
}

/**
 * Rules actions
 */

/**
 * Implementation of hook_rules_action_info().
 */
function simple_contest_rules_action_info() {
  return array(
    'simple_contest_rules_action_change_status' => array(
      'label' => t('Change the contest status'),
      'arguments' => array(
        'contest' => array('type' => 'node', 'label' => t('Contest node')),
      ),
      'module' => 'Simple contest',
      'eval input' => array('status'),
    ),
    'simple_contest_rules_action_finalize_results' => array(
      'label' => t('Complete the contest and finalize the winners'),
      'arguments' => array(
          'contest' => array('type' => 'node', 'label' => t('Contest node'))
      ),
      'module' => 'Simple contest',
    ),
    'simple_contest_rules_action_send_mail_to_winners' => array(
      'label' => t('Send E-mail to all winners'),
      'arguments' => array(
          'contest' => array('type' => 'node', 'label' => t('Contest node')),
          'subject' => array('type' => 'string', 'label' => t('E-mail subject')),
		  'from' => array('type' => 'string', 'label' => t('E-mail from')),
		  'message' => array('type' => 'string', 'label' => t('Message')),
      ),
      'help' => t('IMPORTANT: disable the default buitin email notifications of simple_contest.'),
      'module' => 'Simple contest',
      'eval input' => array('subject', 'message', 'from'),
    ),
    'simple_contest_rules_action_create_new_contest' => array(
      'label' => t('Create new contest'),
      'arguments' => array(
        'contest_type' => array('type' => 'contest_type', 'label' => t('Contest content type')),    	
        'title' => array('type' => 'string', 'label' => t('Contest title')),
        'body' => array('type' => 'string', 'label' => t('Contest body')),
		'state_date' => array('type' => 'date', 'label' => t('Contest start date')),
    	'end_date' => array('type' => 'date', 'label' => t('Contest end date')),
    	'node_user' => array('type' => 'user', 'label' => t('Contest author')),
    	'status' => array('type' => 'string', 'label' => t('Contest status')),
      ),
      'new variables' => array(
        'created_contest' => array(
          'type' => 'node',
          'label' => t('New contest'),
        ),
      ),
      'module' => 'Simple contest',
      'eval input' => array('title', 'body', 'status'),
    ),
  );
}

/**
 * Rules action - simple_contest_rules_action_contest_completed.
 */
function simple_contest_rules_action_finalize_results(&$contest, $settings) {
  
  simple_contest_finalize_results_node($contest);
  
}

/**
 * Rules action - simple_contest_rules_action_contest_completed.
 */
function simple_contest_rules_action_change_status(&$contest, $settings) {  
  $contest->simple_contest['status'] = $settings['status'];
  node_save($contest);  
}

/**
 * Rules action form - simple_contest_rules_action_change_status.
 */
function simple_contest_rules_action_change_status_form($settings, &$form) {
  $form['settings']['status'] = array(
    '#type' => 'select',
    '#title' => t('Change the contest status to'),
    '#options' => simple_contest_status_array(),
    '#default_value' => isset($settings['status']) ? $settings['status'] : array(SIMPLE_CONTEST_STATUS_ACTIVE),
  );
}

function simple_contest_rules_action_change_status_form_submit(&$settings, $form, $form_state) {
  $settings['status'] = array_filter(array_keys(array_filter($settings['status'])));
}


function simple_contest_rules_action_create_new_contest_form($settings, &$form) {
  $form['settings']['status'] = array(
    '#type' => 'select',
    '#title' => t('Contest status'),
    '#options' => simple_contest_status_array(),
    '#default_value' => isset($settings['status']) ? $settings['status'] : array(SIMPLE_CONTEST_STATUS_ACTIVE),
  );
}

function simple_contest_rules_action_create_new_contest_form_submit(&$settings, $form, $form_state) {
  $settings['status'] = array_filter(array_keys(array_filter($settings['status'])));
}

function simple_contest_rules_action_send_mail_to_winners_form($settings, &$form) {
  $settings += array('message' => '');
  $form['settings']['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The message that should be displayed.'),
  );
}

/**
 * Rules action - simple_contest_rules_action_contest_completed.
 */

function simple_contest_rules_action_send_mail_to_winners($contest, $subject, $from, $message, $settings) {

  global $language;

  $from = ($from) ? str_replace(array("\r", "\n"), '', $from) : NULL;
  
  $winners = simple_contest_get_node_result($contest);
  $settings = simple_contest_get_settings($contest->type);
  
  foreach($winners as $info) {
		
	  $account = user_load(array('uid' => $info['uid']));
      if ($info['nid']){
        $participant_node = node_load($info['nid']);
      }else{
        $participant_node = NULL;
      }
      
      $params = array(
        'contest_node' => $contest, 
        'participant_user' => $account, 
        'participant_node' => $participant_node, 
        'settings' => $settings, 
        'participant_rank' => $info['weight'], 
        'subject' => $subject,
        'from' => $from,
        'message' => $message,
      );
      
      drupal_mail('simple_contest_rules', 'rules_action_send_mail_to_winners', $info['mail'], $language, $params, $from);  
    
  }
  
}

/**
 * Rules action - simple_contest_rules_action_contest_completed.
 */
function simple_contest_rules_action_create_new_contest($contest_type, $title, $body, $state_date, $end_date, $node_user, $status) {
  
  $settings = simple_contest_get_settings($contest_type);
  
  $node = new StdClass();
  $date_field = $settings['date_field'];
  $node->type = $contest_type;
  
  $node->title = t($title, array('!start' => $state_date, '!end' => $end_date));
  $node->body = $body;
  
  $node->is_new = TRUE;
  $node->status = 1; // published
  
  $node->simple_contest['status'] = $status;
  $node->uid = $node_user->uid;
  
  // date fields
  $date_field = $settings['date_field'];
  $node->{$date_field}[0]['value'] = $state_date;
  $node->{$date_field}[0]['value2'] = $end_date;

  node_save($node); // save the new node
  
  return array('created_contest' => $node);
  
}


/**
 * Rules Conditions
 */

/**
 * Implementation of hook_rules_condition_info().
 */
function simple_contest_rules_rules_condition_info() {
  return array(
    'simple_contest_rules_is_contest' => array(
      'label' => t('Check the node is a contest type'),
      'arguments' => array(
        'node' => array('type' => 'node', 'label' => t('Contest node'))
      ),
      'module' => 'Simple contest',
    ),
    'simple_contest_rules_contest_hasstatus' => array(
      'label' => t('Contest has statuses'),
      'arguments' => array(
        'node' => array('type' => 'node', 'label' => t('Contest node')),
      ),
      'help' => t('Whether the contest has the selected statuses.'),
      'module' => 'Simple contest',
    ),
  );
}

/**
 * Rules Condition is_contest
 */
function simple_contest_rules_is_contest($node) {
  return simple_contest_is_simple_contest($node);
}

/**
 * Condition user: condition to check whether user has particular roles
 */
function simple_contest_rules_contest_hasstatus($contest, $settings) {
  $statuses = $settings['status'];
  foreach($statuses as $status){
    if ($contest->simple_contest['status'] == $status){
      return TRUE;
    }
  }
  return FALSE;
  
}

/**
 * Condition User: form to select contest status
 */
function simple_contest_rules_contest_hasstatus_form($settings, &$form) {
  $form['settings']['status'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Match the contest status against the selected statuses'),
    '#options' => simple_contest_status_array(),
    '#default_value' => isset($settings['status']) ? $settings['status'] : array(SIMPLE_CONTEST_STATUS_ACTIVE),
  );
}

function simple_contest_rules_contest_hasstatus_submit(&$settings, $form, $form_state) {
  $settings['status'] = array_filter(array_keys(array_filter($settings['status'])));
}



/* Class simple_contest_rules_data_type_contest_type */

/**
* Rules string data type
*/
class simple_contest_rules_data_type_contest_type extends rules_data_type {

  /**
   * Gets the input form for the data
   * Implement it, if your data is not identifiable.
   */
  function get_default_input_form($info, $value) {

    $options = array();
    foreach (simple_contest_get_types() as $node_type => $node_info) {  	
		$options[$node_type] = t($node_info->name);
	}
	
    return array(
      '#type' => 'select',
      '#title' => $info['label'],
      '#description' => t('Simple contest content type'),
      '#required' => TRUE,
      '#options' => $options,
      '#default_value' => $value,
    );
  }
}


/* Class simple_contest_rules_data_type_contest_status */

/**
* Rules string data type
*/
class simple_contest_rules_data_type_contest_status extends rules_data_type {

  function save() {
    return TRUE;
  }

  function load($sid) {
	$options = simple_contest_status_array();
    return $options[$sid];
  }

  function get_identifier() {
    $status = &$this->get();
    $options = array(
	  t('Disabled') => SIMPLE_CONTEST_STATUS_INACTIVE,
	  t('Active') => SIMPLE_CONTEST_STATUS_ACTIVE,
	  t('Canceled') => SIMPLE_CONTEST_STATUS_CANCELED,
	  t('Completed') => SIMPLE_CONTEST_STATUS_COMPLETED,
	 );
    
    return $options[$status];
  }
  
  /**
   * Gets the input form for the data
   * Implement it, if your data is not identifiable.
   */
  function get_default_input_form($info, $value) {

    return array(
      '#type' => 'checkboxes',
      '#title' => $info['label'],
      '#options' => simple_contest_status_array(),
      '#default_value' => $value,
	);
  }
}
